@@ -34,6 +34,8 @@ Node *newNode()
     n->cancelled = 0;
     n->played = 0;
     n->frombutton = 0;
+    n->entry = -1;
+    n->loop = 0;
     n->bytes = 0;
     n->totalbytes = 0;
     n->cachebytes = 0;
@@ -42,6 +44,7 @@ Node *newNode()
     n->play_x = 0;
     n->play_y = 0;
     n->copy = 0;
+    n->prefer_url = 0;
     n->localcache = NULL;
     n->area = NULL;
     n->next = NULL;
@@ -184,10 +187,12 @@ void deleteList(Node * l)
 	    printf("size: %li\n", l->bytes);
 	    printf("remove: %i\n", l->remove);
 	    printf("play: %i\n", l->play);
+	    printf("played: %i\n", l->played);
 	    printf("playlist: %i\n", l->playlist);
 	    printf("speed: %i\n", l->speed);
 	    printf("cancelled: %i\n", l->cancelled);
 	    printf("retrieved: %i\n", l->retrieved);
+	    printf("entry: %i\n", l->entry);
 	    printf("mms stream: %i\n", l->mmsstream);
 	    printf("cache bytes: %li\n", l->cachebytes);
 	    printf("Actual Size: %ix%i\n", l->actual_x, l->actual_y);
@@ -226,7 +231,7 @@ void addToEnd(Node * l, Node * newnode)
 }
 
 void addToList(nsPluginInstance * instance, char *item, Node * parent,
-	       int speed)
+	       int speed, int mmsstream, int entry, int loop)
 {
     Node *node = NULL;
     Node *n;
@@ -265,22 +270,32 @@ void addToList(nsPluginInstance * instance, char *item, Node * parent,
 	    node->mmsstream = 1;
 	} else {
 	    filename = getURLFilename(localitem);
-	    if (instance->keep_download == 1 && filename != NULL) {
+/*	    if (instance->keep_download == 1 && filename != NULL) {
 		snprintf(node->fname, sizeof(node->fname), "%s/%s",
 			 instance->download_dir, filename);
 		if (DEBUG)
 		    printf("Assigned name: %s\n", node->fname);
 		node->remove = 0;
 	    }
+*/
 	    if (filename != NULL)
 		NPN_MemFree(filename);
 	    NPN_GetURLNotify(instance->mInstance, node->url, NULL, NULL);
 	}
+	if (mmsstream == 1)
+	    node->mmsstream = 1;
+	node->entry = entry;
+        node->loop = loop;
+
     } else {
 	if (n->playlist == 1)
 	    n->mmsstream = 1;
+	n->entry = entry;
+        n->loop = loop;
     }
 
+
+
 }
 
 void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
@@ -291,14 +306,14 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
     char url[1024];
     unsigned long int size, found, i;
     long int remainder;
-    char *p;
+    char *p, *entry, *href, *repeat;
     char *nextrmda = NULL;
     char *rdrf;
     char *rmdr;
     char *sub, *refurl;
     char *seqtag, *rate, *videotag, *videosrc;
     unsigned int code;
-    int speed;
+    int speed, entry_counter = -1, loopentry;
     unsigned char *c1, *c2, *c3, *c4;
 
     fp = fopen(file, "r");
@@ -383,7 +398,8 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 				printf("Skipped URL: %s\n", refurl);
 			} else {
 			    parent->playlist = 1;
-			    addToList(instance, refurl, parent, speed);
+			    addToList(instance, refurl, parent, speed, 0,
+				      -1, 0);
 			    printf("ADDED URL: %s\n", (refurl));
 			    printf("code: %i\n speed %i\n", code, speed);
 			    found = 1;
@@ -489,7 +505,8 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 				if (DEBUG)
 				    printf("[smil] found url %s\n", url);
 				parent->playlist = 1;
-				addToList(instance, url, parent, speed);
+				addToList(instance, url, parent, speed, 0,
+					  -1, 0);
 				found = 1;
 			    } else {
 				seqtag = (char *) NULL;
@@ -554,7 +571,7 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			    if (DEBUG)
 				printf("URL: %s\n", url);
 			    parent->playlist = 1;
-			    addToList(instance, url, parent, -1);
+			    addToList(instance, url, parent, -1, 0, -1, 0);
 			    p = p + strlen(url) + 1;
 			    found = 1;
 			} else {
@@ -571,6 +588,113 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 		    printf("found MP3\n");
 		break;
 	    }
+	    // asx version 3.0 files
+	    // <Ref href = "url" />
+	    snprintf((char *) buffer_lower, sizeof(buffer_lower), "%s",
+		     buffer);
+	    lowercase((char *) buffer_lower);
+	    p = buffer_lower;
+	    if ((memmem
+		 (p, size, "<asx version=\"3.0\">",
+		  strlen("<asx version=\"3.0\">"))
+		 || memmem(p, size, "<asx version = \"3.0\">",
+			   strlen("<asx version = \"3.0\">")))
+		&& found == 0) {
+                    parent->playlist = 1;
+                    parent->prefer_url = 1;
+                    found = 1;
+
+/*
+		if (DEBUG)
+		    printf("found ASX 3.0file\n");
+		while (p != NULL) {
+		    i = size - ((long) p - (long) buffer_lower);
+
+		    if (i < 1)
+			break;
+                    repeat = (char *) memmem(p, i, "<repeat", 7);
+		    entry = (char *) memmem(p, i, "entry", 5);
+		    href = (char *) memmem(p, i, "href", 4);
+		    if (entry == NULL || href == NULL) {
+			// do nothing
+		    } else {
+			if (entry < href) {
+			    entry_counter =
+				highest_entry(instance->list) + 1;
+			    if (DEBUG)
+				printf("incrementing entry counter\n");
+			}
+                        loopentry = 0;
+                        if (repeat != NULL) {
+                            if (repeat < entry && (char *) memmem(p, i, "</repeat", 8) > entry) {
+                                loopentry = 1;
+                            }
+                        }
+		    }
+
+		    p = (char *) memmem(p, i, "href", 4);
+		    if (p == NULL)
+			break;
+		    if ((p - 9) > buffer_lower) {
+			// look backwards in the file and see if the href is preceeded by
+			// a qualifier tag, but don't look too far forward otherwise we may
+			// find another tag of this type
+			if (memmem(p - 5, 5, "logo", 4) != NULL) {
+			    if (DEBUG > 1)
+				printf("found logo in ASX\n");
+			    p = p + 4;
+			    continue;
+			}
+			if (memmem(p - 7, 7, "banner", 6) != NULL) {
+			    if (DEBUG > 1)
+				printf("found banner in ASX\n");
+			    p = p + 4;
+			    continue;
+			}
+			if (memmem(p - 9, 9, "moreinfo", 8) != NULL) {
+			    if (DEBUG > 1)
+				printf("found moreinfo in ASX\n");
+			    p = p + 4;
+			    continue;
+			}
+
+			if (memmem(p - 9, 9, "entryref", 8) != NULL) {
+			    if (DEBUG > 1)
+				printf("found entryref in ASX\n");
+			    //p = p + 4;
+			    //continue;
+			    //entry_counter = highest_entry(instance->list) + 1;
+			}
+
+		    }
+		    p = p + 4;
+		    p = strchr(p, '\"');
+		    if (p != NULL) {
+			p++;
+			i = (long) p - (long) buffer_lower;
+			if (i >= size)
+			    break;
+			snprintf(url, 1024, "%s", buffer + i);
+
+			sub = strchr(url, '\"');
+			if (sub != NULL) {
+			    *sub = '\0';
+			    unEscapeXML(url);
+			    if (DEBUG)
+				printf("URL: %s\n", url);
+			    parent->playlist = 1;
+			    addToList(instance, url, parent, -1, 1,
+				      entry_counter,loopentry);
+			    p = p + strlen(url) + 1;
+			    found = 1;
+			} else {
+			    break;
+			}
+		    }
+		}
+*/
+		break;
+	    }
 	    // asx files are XMLish
 	    // <Ref href = "url" />
 	    snprintf((char *) buffer_lower, sizeof(buffer_lower), "%s",
@@ -580,6 +704,10 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 	    if (memmem(p, size, "<asx", strlen("<asx")) && found == 0) {
 		if (DEBUG)
 		    printf("found ASX file\n");
+                    parent->playlist = 1;
+                    parent->prefer_url = 1;
+                    found = 1;
+/*
 		while (p != NULL) {
 		    i = size - ((long) p - (long) buffer_lower);
 
@@ -623,10 +751,11 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			sub = strchr(url, '\"');
 			if (sub != NULL) {
 			    *sub = '\0';
+			    unEscapeXML(url);
 			    if (DEBUG)
 				printf("URL: %s\n", url);
 			    parent->playlist = 1;
-			    addToList(instance, url, parent, -1);
+			    addToList(instance, url, parent, -1, 1, -1,0);
 			    p = p + strlen(url) + 1;
 			    found = 1;
 			} else {
@@ -634,6 +763,7 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			}
 		    }
 		}
+*/
 		break;
 	    }
 	    // asf files
@@ -666,7 +796,8 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			    *sub = '\0';
 			    if (DEBUG)
 				printf("URL: %s\n", url);
-			    addToList(instance, url, parent, -1);
+			    // with these files they are mirror sites when multiple references exist
+			    addToList(instance, url, parent, -1, 0, 0,0);
 			    parent->playlist = 1;
 			    if (parent->mmsstream == 1
 				&& parent->next == NULL)
@@ -695,29 +826,40 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 		parent->cancelled = 1;
 		found = 1;
 	    }
+	    // we have an SCP Playlist file
+	    if (memmem(buffer_lower, size, "[playlist]", 10) != NULL) {
+		parent->mmsstream = 1;
+		parent->cancelled = 1;
+		found = 1;
+	    }
 	    // simple playlist usually realmedia file
-	    if (strncasecmp(buffer, "rtsp://", 7) == 0 && found == 0) {
-		p = buffer;
+	    if (strncasecmp(buffer, "rtsp", 4) == 0 && found == 0) {
+		if (DEBUG) {
+		    printf("Search RTSP File\n");
+		}
+		p = buffer_lower;
 		while (p != NULL) {
-		    i = size - ((long) p - (long) buffer);
+		    i = size - ((long) p - (long) buffer_lower);
 		    p = (char *) memmem(p, i, "rtsp://", 7);
 		    if (p == NULL) {
 			break;
 		    } else {
-			snprintf(url, 1024, "%s", p);
-			sub = strchr(url, 0xd);	   // terminate on CR
+			i = (long) p - (long) buffer_lower;
+			snprintf(url, 1024, "rtsp://%s", buffer + i + 7);
+			sub = strchr(url, 0xd);	// terminate on CR
 			if (sub != NULL)
 			    sub[0] = '\0';
-			sub = strchr(url, 0xa);	   // terminate on line feed
+			sub = strchr(url, 0xa);	// terminate on line feed
 			if (sub != NULL)
 			    sub[0] = '\0';
-			sub = strchr(url, 0x20);   // terminate on space
+			//sub = strchr(url, 0x20);	// terminate on space
+			sub = strchr(url, EOF);	// terminate on EOF
 			if (sub != NULL)
 			    sub[0] = '\0';
 			if (DEBUG)
 			    printf("URL: %s\n", url);
 			parent->playlist = 1;
-			addToList(instance, url, parent, -1);
+			addToList(instance, url, parent, -1, 0, -1,0);
 			p = p + strlen(url) + 1;
 			found = 1;
 		    }
@@ -742,13 +884,13 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			sub = strchr(url, 0xa);
 			if (sub != NULL)
 			    sub[0] = '\0';
-			sub = strchr(url, 0x20);   // terminate on space
+			sub = strchr(url, 0x20);	// terminate on space
 			if (sub != NULL)
 			    sub[0] = '\0';
 			if (DEBUG)
 			    printf("URL: %s\n", url);
 			parent->playlist = 1;
-			addToList(instance, url, parent, -1);
+			addToList(instance, url, parent, -1, 0, -1,0);
 			p = p + strlen(url) + 1;
 			found = 1;
 		    }
@@ -770,13 +912,13 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			sub = strchr(url, 0xa);
 			if (sub != NULL)
 			    sub[0] = '\0';
-			sub = strchr(url, 0x20);   // terminate on space
+			sub = strchr(url, 0x20);	// terminate on space
 			if (sub != NULL)
 			    sub[0] = '\0';
 			if (DEBUG)
 			    printf("URL: %s\n", url);
 			parent->playlist = 1;
-			addToList(instance, url, parent, -1);
+			addToList(instance, url, parent, -1, 0, -1,0);
 			p = p + strlen(url) + 1;
 			found = 1;
 		    }
@@ -879,6 +1021,7 @@ void printNode(Node * l)
     printf("remove: %i\n", l->remove);
     printf("play: %i\n", l->play);
     printf("playlist: %i\n", l->playlist);
+    printf("entry: %i\n", l->entry);
     printf("speed: %i\n", l->speed);
     printf("cancelled: %i\n", l->cancelled);
     printf("retrieved: %i\n", l->retrieved);
@@ -901,3 +1044,48 @@ void printList(Node * l)
 	current = current->next;
     }
 }
+
+void markPlayed(Node * l)
+{
+    Node *n;
+
+    if (DEBUG)
+	printf("markPlayed\n");
+
+    if (l->entry == -1) {
+	if (DEBUG)
+	    printf("marking %s as played\n", l->url);
+	l->played = 1;
+    } else {
+	n = l;
+	while (n != NULL) {
+	    if (n->entry == l->entry) {
+		if (DEBUG)
+		    printf("marking %s as played\n", n->url);
+		n->played = 1;	// set played flag for all mirror lists
+	    }
+	    n = n->next;
+	}
+    }
+
+}
+
+int highest_entry(Node * l)
+{
+
+    Node *n;
+    int entry = -1;
+
+    if (DEBUG)
+	printf("highest_entry\n");
+
+    n = l;
+    while (n != NULL) {
+	if (n->entry > entry) {
+	    entry = n->entry;
+	}
+	n = n->next;
+    }
+
+    return entry;
+}
