@@ -345,7 +345,8 @@ char *GetMIMEDescription()
 		    "audio/x-mpeg:mpg,mpeg:MPEG;"
 		    "audio/mpeg2:mp2:MPEG audio;"
 		    "audio/x-mpeg2:mp2:MPEG audio;"
-		    "video/mp4:mp4:MPEG 4 Video;", sizeof(MimeTypes));
+		    "video/mp4:mp4:MPEG 4 Video;"
+		    "video/3gpp:mp4,3gp:MPEG 4 Video;", sizeof(MimeTypes));
 
 	    if (enable_mp3)
 		strlcat(MimeTypes,
@@ -360,14 +361,24 @@ char *GetMIMEDescription()
 	    strlcat(MimeTypes,
 		    "application/x-ogg:ogg:Ogg Vorbis Media;"
 		    "audio/ogg:ogg:Ogg Vorbis Audio;"
+		    "audio/x-ogg:ogg:Ogg Vorbis Audio;"
 		    "application/ogg:ogg:Ogg Vorbis / Ogg Theora;",
 		    sizeof(MimeTypes));
 	}
+	// FLAC
+	strlcat(MimeTypes,
+		"audio/flac:flac:FLAC Audio;"
+		"audio/x-flac:flac:FLAC Audio;", sizeof(MimeTypes));
+
 	// FLI
 	strlcat(MimeTypes,
 		"video/fli:fli,flc:FLI animation;"
 		"video/x-fli:fli,flc:FLI animation;", sizeof(MimeTypes));
 
+	// FLV
+	strlcat(MimeTypes, "video/x-flv:flv:Flash Video;",
+		sizeof(MimeTypes));
+
 	// Vivo
 	strlcat(MimeTypes, "video/vnd.vivo:viv,vivo:VivoActive;",
 		sizeof(MimeTypes));
@@ -379,10 +390,9 @@ char *GetMIMEDescription()
 
 	// Soundtracker
 	strlcat(MimeTypes,
-		"audio/x-mod:mod:Soundtracker;",
-		sizeof(MimeTypes));
-    
- 	// Basic
+		"audio/x-mod:mod:Soundtracker;", sizeof(MimeTypes));
+
+	// Basic
 	strlcat(MimeTypes,
 		"audio/basic:au,snd:Basic Audio File;"
 		"audio/x-basic:au,snd:Basic Audio File;",
@@ -411,16 +421,16 @@ char *GetMIMEDescription()
 #endif
 
 #ifdef DVX
-       // DIVX
-    	if (enable_dvx) {
-	strlcat(MimeTypes,
-		"video/divx:divx:DivX Media Format;"
-		"video/vnd.divx:divx:DivX Media Format;",
-		sizeof(MimeTypes));
+	// DIVX
+	if (enable_dvx) {
+	    strlcat(MimeTypes,
+		    "video/divx:divx:DivX Media Format;"
+		    "video/vnd.divx:divx:DivX Media Format;",
+		    sizeof(MimeTypes));
 	}
-#endif    
-    
-    
+#endif
+
+
     }
     ret = strdup(MimeTypes);
     if (DEBUG)
@@ -559,6 +569,10 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		    || strstr(parameters->argv[i], "1")) {
 		    instance->nomediacache = 1;
 		}
+		if (DEBUG)
+		    printf
+			("No media cache is set to %i in the embed tag\n",
+			 instance->nomediacache);
 	    }
 
 	    if (strncasecmp(parameters->argn[i], "src", 3) == 0) {
@@ -672,6 +686,25 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		if (DEBUG)
 		    printf("hidden=%i\n", instance->hidden);
 	    }
+
+	    if (strncasecmp
+		(parameters->argn[i], "displaysize",
+		 strlen("displaysize")) == 0) {
+
+		lowercase(parameters->argv[i]);
+		if (DEBUG)
+		    printf("argv[i]=%s\n", parameters->argv[i]);
+
+		if (strstr(parameters->argv[i], "true")
+		    || strstr(parameters->argv[i], "yes")
+		    || strstr(parameters->argv[i], "1")) {
+		    instance->dontparent = 1;
+		} else {
+		    instance->dontparent = 0;
+		}
+		if (DEBUG)
+		    printf("targetplayer=%i\n", instance->targetplayer);
+	    }
 	    // target is used by Google Video to specifiy the URL to be played
 	    // so we might have to do something here.
 	    if (strncasecmp(parameters->argn[i], "target", 6) == 0) {
@@ -830,7 +863,8 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		    printf("showlogo=%i\n", instance->showlogo);
 	    }
 
-	    if (strncasecmp(parameters->argn[i], "enablecontextmenu", 8) == 0) {
+	    if (strncasecmp(parameters->argn[i], "enablecontextmenu", 8) ==
+		0) {
 
 		lowercase(parameters->argv[i]);
 		if (DEBUG)
@@ -844,7 +878,8 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		    instance->enablecontextmenu = 1;
 		}
 		if (DEBUG)
-		    printf("enablecontextmenu=%i\n", instance->enablecontextmenu);
+		    printf("enablecontextmenu=%i\n",
+			   instance->enablecontextmenu);
 	    }
 
 	    if ((strncasecmp(parameters->argn[i], "controls", 8) == 0)
@@ -1097,6 +1132,43 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 			   instance->onDestroyCallback);
 	    }
 
+	    if (strncasecmp
+		(parameters->argn[i], "tvdriver",
+		 strlen("tvdriver")) == 0) {
+		instance->tv_driver = strdup(parameters->argv[i]);
+		if (DEBUG)
+		    printf("tv_driver=%s\n", instance->tv_driver);
+	    }
+
+	    if (strncasecmp
+		(parameters->argn[i], "tvinput", strlen("tvinput")) == 0) {
+		instance->tv_input = strdup(parameters->argv[i]);
+		if (DEBUG)
+		    printf("tv_input=%s\n", instance->tv_input);
+	    }
+
+	    if (strncasecmp
+		(parameters->argn[i], "tvheight",
+		 strlen("tvheight")) == 0) {
+		if (strstr(parameters->argv[i], "%") == NULL)
+		    sscanf(parameters->argv[i], "%i",
+			   &instance->tv_height);
+	    }
+
+	    if (strncasecmp
+		(parameters->argn[i], "tvwidth", strlen("tvwidth")) == 0) {
+		if (strstr(parameters->argv[i], "%") == NULL)
+		    sscanf(parameters->argv[i], "%i", &instance->tv_width);
+	    }
+
+	    if (strncasecmp
+		(parameters->argn[i], "tvoutfmt",
+		 strlen("tvoutfmt")) == 0) {
+		instance->tv_outfmt = strdup(parameters->argv[i]);
+		if (DEBUG)
+		    printf("tv_outfmt=%s\n", instance->tv_outfmt);
+	    }
+
 	    if (instance->nQtNext < 256
 		&& (strncasecmp(parameters->argn[i], "qtnext", 6) == 0)
 		&& parameters->argv[i][0] == '<') {
@@ -1283,6 +1355,11 @@ void LoadConfigFile(nsPluginInstance * instance)
 		    sscanf(parse, "%i", &instance->nomediacache);
 		    if (instance->nomediacache != 0)
 			instance->nomediacache = 1;
+		    if (DEBUG)
+			printf
+			    ("No media cache is set to %i in conf file\n",
+			     instance->nomediacache);
+
 		    continue;
 		}
 
@@ -1474,6 +1551,15 @@ void LoadConfigFile(nsPluginInstance * instance)
 		    continue;
 		}
 
+		if (strncasecmp(buffer, "softvol", strlen("softvol")) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->softvol);
+		    if (instance->softvol != 0)
+			instance->softvol = 1;
+		    continue;
+		}
+
 		if (strncasecmp
 		    (buffer, "nopauseonhide",
 		     strlen("nopauseonhide")) == 0) {
@@ -1514,6 +1600,70 @@ void LoadConfigFile(nsPluginInstance * instance)
 			instance->useragent = strdup(parse);
 		    continue;
 		}
+
+		if (strncasecmp(buffer, "tv-driver", strlen("tv-driver"))
+		    == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "=\n"));
+		    if (instance->tv_driver != NULL)
+			free(instance->tv_driver);
+		    if (strstr(parse, "`") == NULL)	// don't allow shell commands
+			instance->tv_driver = strdup(parse);
+		    continue;
+		}
+
+		if (strncasecmp(buffer, "tv-input", strlen("tv-input")) ==
+		    0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "=\n"));
+		    if (instance->tv_input != NULL)
+			free(instance->tv_input);
+		    if (strstr(parse, "`") == NULL)	// don't allow shell commands
+			instance->tv_input = strdup(parse);
+		    continue;
+		}
+
+		if (strncasecmp(buffer, "tv-height", strlen("tv-height"))
+		    == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->tv_height);
+		    if (instance->tv_height < 0)
+			instance->tv_height = 0;
+		    continue;
+		}
+
+		if (strncasecmp(buffer, "tv-width", strlen("tv-width")) ==
+		    0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->tv_width);
+		    if (instance->tv_width < 0)
+			instance->tv_width = 0;
+		    continue;
+		}
+
+		if (strncasecmp(buffer, "tv-outfmt", strlen("tv-outfmt"))
+		    == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "=\n"));
+		    if (instance->tv_outfmt != NULL)
+			free(instance->tv_outfmt);
+		    if (strstr(parse, "`") == NULL)	// don't allow shell commands
+			instance->tv_outfmt = strdup(parse);
+		    continue;
+		}
+
+		if (strncasecmp(buffer, "profile", strlen("profile"))
+		    == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "=\n"));
+		    if (instance->profile != NULL)
+			free(instance->profile);
+		    if (strstr(parse, "`") == NULL)	// don't allow shell commands
+			instance->profile = strdup(parse);
+		    continue;
+		}
 		// SMIL
 		if (strncasecmp(buffer, "enable-smil", 11) == 0) {
 		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
