@@ -171,6 +171,7 @@ FILE *mypopen(char **argv, pid_t * pid, int *control,
     pid_t child;
     long flags;
     sigset_t newmask;
+    FILE *retfd;
 
     pipe(filedesr);
     pipe(filedesw);
@@ -232,7 +233,9 @@ FILE *mypopen(char **argv, pid_t * pid, int *control,
 #endif
 	fcntl(*control, F_SETFL, flags);
 
-	return fdopen(filedesr[0], "r");
+	retfd = fdopen(filedesr[0], "r");
+
+	return retfd;
     }
 }
 
@@ -394,8 +397,17 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	    instance->td->argv[i++] = strdup(buffer);
 #endif
 #ifdef GTK_ENABLED
-	    if (instance->targetplayer == 1)
-		gtk_widget_show(instance->gtkwidget);
+//          if (instance->targetplayer == 1) {
+//              gtk_widget_show(instance->gtkwidget);
+//          }
+
+	    if (GTK_IS_WIDGET(instance->drawing_area)) {
+		gtk_widget_show(instance->drawing_area);
+		gtk_widget_set_usize(instance->drawing_area, 1, 1);
+		while (gtk_events_pending())
+		    gtk_main_iteration();
+
+	    }
 #ifdef GTK2_ENABLED
 	    instance->player_window =
 		gtk_socket_get_id(GTK_SOCKET(instance->drawing_area));
@@ -415,7 +427,7 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 #endif
 	    if (instance->targetplayer == 1)
 		gtk_widget_hide(instance->gtkwidget);
-	    
+
 #endif
 	} else {
 	    instance->player_window = 0;
@@ -433,6 +445,7 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 #ifdef GTK_ENABLED
 	    gtk_widget_set_usize(GTK_WIDGET(instance->status),
 				 instance->window_width - 20, 19);
+	    gtk_widget_show(instance->drawing_area);
 #ifdef GTK2_ENABLED
 	    instance->player_window =
 		gtk_socket_get_id(GTK_SOCKET(instance->drawing_area));
@@ -529,6 +542,13 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	instance->td->argv[i++] = strdup(buffer);
     }
 
+    if (instance->profile) {
+	snprintf(buffer, 1024, "-profile");
+	instance->td->argv[i++] = strdup(buffer);
+	snprintf(buffer, 1024, "%s", instance->profile);
+	instance->td->argv[i++] = strdup(buffer);
+    }
+
     if (instance->af) {
 	snprintf(buffer, 1024, "-af");
 	instance->td->argv[i++] = strdup(buffer);
@@ -581,6 +601,10 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
     }
 //    snprintf(buffer, 1024, "-nofs");  /* no full screen */
 //    instance->td->argv[i++] = strdup(buffer);
+    if (instance->softvol) {
+	snprintf(buffer, 1024, "-softvol");	/* use software volume control */
+	instance->td->argv[i++] = strdup(buffer);
+    }
     snprintf(buffer, 1024, "-slave");	/* slave mode */
     instance->td->argv[i++] = strdup(buffer);
     instance->td->argv[i++] = NULL;
@@ -704,8 +728,8 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
     char *eos;
     char *msg;
 
-    long cfbytes;
-    int i;
+    long cfbytes, flags = 0;
+    int i, amt;
     int zerocfbytes_count = 0;
     char url_copy[1024];
     int c;
@@ -750,6 +774,13 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
     local_td->instance->movie_height = 0;
     local_td->instance->movie_width = 0;
 
+    if (local_td->instance->player != NULL) {
+	if (DEBUG) {
+	    printf("Getting file mode flags\n");
+	}
+	flags = fcntl(fileno(local_td->instance->player), F_GETFL, 0);
+    }
+
     while (1) {
 	pthread_testcancel();
 	if (local_td->instance->player == NULL)
@@ -774,10 +805,9 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 
 	pthread_testcancel();
 	if (local_td->instance->player != NULL) {
-
-	    //if (fgets(buffer, 1024, local_td->instance->player) == NULL) {
+	    // if (fgets(buffer, 1024, local_td->instance->player) == NULL) {
 	    //  continue;
-	    //}
+	    // }
 
 	    // fgets is not a pthread cancel point, so we basically have to rewrite fgets
 	    // to make this work better. If we cancel the thread while fgets is waiting for 
@@ -792,9 +822,10 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		if ((local_td->instance->cancelled == 0)
 		    && (local_td->instance->player != NULL)) {
 		    // c = fgetc(local_td->instance->player);
-		    fread(&c, 1, 1, local_td->instance->player);
+		    amt = fread(&c, 1, 1, local_td->instance->player);
 		} else {
 		    c = EOF;
+		    amt = 0;
 		}
 		pthread_mutex_unlock(&(local_td->instance->read_mutex));
 
@@ -805,7 +836,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		if (c == 0)
 		    continue;
 		buffer[i] = (char) c;
-		i++;
+		i += amt;
 		if (i >= 1024) {
 		    buffer[1023] = '\0';
 		    break;
@@ -815,14 +846,42 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		continue;
 	    else if (i < 1024)	// make sure we don't access out of bounds
 		buffer[i] = '\0';	// make sure we NULL terminate the string
+
 	}
+
 	pthread_testcancel();
 
 	if (DEBUG) {
-	    printf("READ: %s \n", buffer);
+	    if (strlen(buffer) > 0)
+		printf("READ: %s \n", buffer);
 	}
 
 	refresh_frame(buffer, local_td, local_list);
+
+
+	pthread_testcancel();
+	if (strstr(buffer, "STREAM") != NULL) {
+	    if (local_td->instance->player != NULL) {
+		if (DEBUG) {
+		    printf("Setting file mode to non-blocking\n");
+		}
+		fcntl(fileno(local_td->instance->player), F_SETFL,
+		      flags | O_NONBLOCK);
+	    }
+	}
+
+	pthread_testcancel();
+	if (strstr(buffer, "Cache size") != NULL
+	    || strstr(buffer, "Starting playback") != NULL) {
+	    if (local_td->instance->player != NULL) {
+		if (DEBUG) {
+		    printf("Setting file mode back to blocking\n");
+		}
+		fcntl(fileno(local_td->instance->player), F_SETFL, flags);
+	    }
+	}
+
+
 	pthread_testcancel();
 	if (strstr(buffer, "Cache fill:") != NULL) {
 	    strlcpy(message, strstr(buffer, "Cache fill"), 1024);
@@ -860,7 +919,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		    printf("Cache Fill: 0%% (0 bytes), count = %d\n",
 			   zerocfbytes_count);
 		}
-		if (zerocfbytes_count >= 1 &&
+		if (zerocfbytes_count >= 3 &&
 		    ((strncmp(local_url, "mms://", 6) == 0) ||
 		     (local_mmsstream
 		      && strncmp(local_url, "http://", 7) == 0))) {
@@ -913,6 +972,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 
 	pthread_testcancel();
 	if (strstr(buffer, "VO:") != NULL) {
+	    fcntl(fileno(local_td->instance->player), F_SETFL, flags);
 	    if (local_td->instance->mode == NP_EMBED
 		&& local_td->instance->noembed == 0)
 		local_td->instance->noredraw = 1;
@@ -972,6 +1032,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 
 	pthread_testcancel();
 	if (strstr(buffer, "Video: no video") != NULL) {
+	    fcntl(fileno(local_td->instance->player), F_SETFL, flags);
 	    if (local_td->instance->panel_height == 0)
 		local_td->instance->panel_height = 32;
 	    local_td->instance->noredraw = 0;
@@ -996,6 +1057,20 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		       local_td->instance->mediaLength);
 	}
 
+	pthread_testcancel();
+	// mplayer answer back messages
+	if (strstr(buffer, "ANS_volume") != 0) {
+	    if ((int) local_td->instance->mediaLength != 0)
+		lastmedialength = local_td->instance->mediaLength;
+	    msg = strstr(buffer, "ANS_volume");
+	    sscanf(msg, "ANS_volume=%i", &(local_td->instance->volume));
+	    if (DEBUG)
+		printf("Volume = %i\n", local_td->instance->volume);
+#ifdef GTK_ENABLED
+	    g_idle_add(gtkgui_setvolumetip, local_td->instance);
+#endif
+	}
+
 	pthread_testcancel();
 	// mplayer answer back messages
 	if (strstr(buffer, "ANS_TIME_POSITION") != 0) {
@@ -1098,6 +1173,9 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 			length_request_count++;
 		    }
 		}
+		if (local_td->instance->volume == -1) {
+		    sendCommand(local_td->instance, "get_property volume");
+		}
 		pthread_mutex_unlock(&(local_td->instance->control_mutex));
 		pthread_cleanup_pop(0);
 #ifdef GTK_ENABLED
@@ -1118,6 +1196,39 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    DrawUI(local_td->instance->widget,
 		   local_td->instance, message, 0, -1);
 #endif
+#ifdef GTK_ENABLED
+	    snprintf(local_td->instance->lastmessage, 1024, "%s", message);
+	    g_idle_add(gtkgui_message, local_td->instance);
+#endif
+	    pthread_mutex_lock(&(local_td->instance->control_mutex));
+	    local_td->instance->js_state = JS_STATE_WAITING;
+	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
+
+	}
+
+/*
+	ICY Info: StreamTitle='Three Days Grace - Animal I Have Become';StreamUrl='http://www.thebuzz.fm';
+*/
+	pthread_testcancel();
+	if (strstr(buffer, "ICY Info") != NULL) {
+	    msg = strstr(buffer, "'");
+	    if (msg != NULL) {
+		strncpy(message, _("Stream Title: "), 1024);
+		strncat(message, msg + 1, 1024);
+		for (i = 0; i < (int) strlen(message) - 1; i++) {
+		    if (message[i] == '\'') {
+			message[i] = '\0';
+			break;
+		    }
+		}
+		if (DEBUG) {
+		    printf("Title: %s\n", message);
+		}
+	    }
+#ifdef X_ENABLED
+	    DrawUI(local_td->instance->widget,
+		   local_td->instance, message, 0, -1);
+#endif
 #ifdef GTK_ENABLED
 	    snprintf(local_td->instance->lastmessage, 1024, "%s", message);
 	    g_idle_add(gtkgui_message, local_td->instance);
@@ -1170,15 +1281,26 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    break;
 	}
 
- /*
+
 	pthread_testcancel();
-	if (strstr(buffer, "File not found") != NULL) {
-	    result->tryagain = TRYAGAIN_FALSE;
-	    result->errorcode = ERROR_FILE_NOT_FOUND;
-	    break;
+	if (strstr(buffer, "Failed to open") != NULL) {
+	    if ((strstr(buffer, "/dev/rtc") == NULL)
+		&& (strstr(buffer, "LIRC") == NULL)
+		&& (strstr(buffer, "registry file") == NULL)) {	// ignore RTC device and LIRC device failures
+		if (local_td->instance->currentnode->mmsstream == 1) {
+		    if (DEBUG)
+			printf("Trying fallback\n");
+		    result->tryagain = TRYAGAIN_FALLBACK;
+		    result->errorcode = ERROR_FILE_NOT_FOUND;
+		    break;
+		} else {
+		    result->tryagain = TRYAGAIN_FALSE;
+		    result->errorcode = ERROR_FILE_NOT_FOUND;
+		    break;
+		}
+	    }
 	}
-*/
-    
+
 	pthread_testcancel();
 	if (strstr(buffer, "Error while decoding") != NULL) {
 	    if (!isMms(local_url, local_td->instance->nomediacache)) {
@@ -1203,6 +1325,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		printf("----player thread: breaking read loop - Quit\n");
 	    local_td->instance->js_state = JS_STATE_UNDEFINED;
 	    result->tryagain = TRYAGAIN_FALSE;
+	    result->errorcode = ERROR_QUIT;
 	    break;
 	}
 
@@ -1294,14 +1417,28 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    if (DEBUG)
 		printf
 		    ("----player thread: breaking read loop - Exiting\n");
-	    if (local_td->instance->mediaPercent != 0) {
+	    if (local_td->instance->mediaPercent == 0) {
+		if (DEBUG)
+		    printf("local_url = %s\nprefer_url = %i\n", local_url,local_list->prefer_url);
+		if (local_list->prefer_url == 1) {
+
+		    result->tryagain = TRYAGAIN_FALSE;
+		    result->errorcode = ERROR_NO_ERROR;
+
+		} else {
+		    result->tryagain = TRYAGAIN_TRUE;
+		    result->errorcode = ERROR_USE_URL;
+		}
+
+	    } else {
 		local_td->instance->mediaPercent = 100;
 #ifdef GTK_ENABLED
 		g_idle_add(gtkgui_drawMediaProgress, local_td->instance);
 #endif
+		result->tryagain = TRYAGAIN_FALSE;
+		result->errorcode = ERROR_NO_ERROR;
 	    }
-	    result->tryagain = TRYAGAIN_FALSE;
-	    result->errorcode = ERROR_NO_ERROR;
+
 	    break;
 	}
 
@@ -1397,7 +1534,6 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 */
 
 	assert(local_td->instance->player != NULL);
-
     }				// end of READ LOOP
 
     if (DEBUG) {
@@ -1413,7 +1549,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
     }
 
     if (result->tryagain == TRYAGAIN_FALLBACK) {
-	//fallback to msst if we can
+	//fallback to msst and then to http if we can
 	strlcpy(url_copy, local_url, 1023);
 	url_copy[1023] = '\0';
 
@@ -1421,13 +1557,16 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    snprintf(local_url, 1023, "mmst://%s", url_copy + 6);
 	    result->retval = TRUE;
 	} else if (local_mmsstream &&
-		   strncmp(local_url, "http://", 7) == 0) {
+		   strncmp(local_url, "mmst://", 7) == 0) {
 
-	    snprintf(local_url, 1023, "mmst://%s", url_copy + 7);
+	    snprintf(local_url, 1023, "http://%s", url_copy + 7);
 	    result->retval = TRUE;
 	} else {
 	    result->retval = FALSE;
 	}
+	if (DEBUG)
+	    printf("TRYAGAIN_FALLBACK entered, new url is %s\n",
+		   local_url);
     }
 #ifdef GTK_ENABLED
     if (strstr(local_td->instance->lastmessage, "Error") == NULL) {
@@ -1465,6 +1604,7 @@ void *playPlaylist(void *td)
     int maybeplaylist;
     int listempty;
     int lasterror = -1;
+    int force_url = 0;
 
     local_td = (ThreadData *) td;
     local_list = local_td->list;
@@ -1606,7 +1746,7 @@ void *playPlaylist(void *td)
 		 i);
 
 	// if the play count is exactly 1 then tell mplayer to loop continuously and to use the same frame buffer (should eliminate blinking)
-	if (i == 1) {
+	if (i == 1 || local_list->loop == 1) {
 	    loop_single = 1;
 	    local_td->argv[argc++] = strdup("-loop");
 	    snprintf(buffer, 32, "%i", local_td->instance->loop);
@@ -1617,10 +1757,13 @@ void *playPlaylist(void *td)
     }
 
     loop_count = local_td->instance->loop;
+    if (DEBUG)
+	printf("----player thread: set loop_count to %i\n", loop_count);
 
     while (1) {
 	if (DEBUG)
-	    printf("----player thread: entering loop\n");
+	    printf("----player thread: entering loop. loop_count = %i\n",
+		   loop_count);
 
 	pthread_cleanup_push((void (*)(void *)) pthread_mutex_unlock,
 			     (void *) &(local_td->instance->
@@ -1635,6 +1778,11 @@ void *playPlaylist(void *td)
 	listempty = 0;
 	local_list = local_td->list;
 	while (local_list != NULL) {
+	    if (DEBUG)
+		printf
+		    ("----player thread: play = %i played = %i entry = %i fname = %s\n",
+		     local_list->play, local_list->played,
+		     local_list->entry, local_list->fname);
 	    if (local_list->play && !(local_list->played)) {
 		break;
 	    }
@@ -1644,11 +1792,22 @@ void *playPlaylist(void *td)
 	if (local_list == NULL) {
 	    //nothing to play: will exit, unless we have loop
 	    // the -- is to countdown the loop
+	    if (DEBUG)
+		printf
+		    ("----player thread: local_list is NULL loop_count = %i\n",
+		     loop_count);
+
 	    if (local_td->instance->loop != 0)
 		loop_count--;
 
+	    if (DEBUG)
+		printf
+		    ("----player thread: loop_count = %i loop_single = %i\n",
+		     loop_count, loop_single);
+
 	    // we're gonna reset the playlist, because we are going to play it again
-	    if ((loop_count >= 0) && (loop_single == 0)) {
+	    if ((loop_count > 0) && (loop_single == 0)
+		|| (local_td->instance->loop == 0)) {
 		local_list = local_td->list;
 		while (local_list != NULL) {
 		    if (local_list->play)
@@ -1728,11 +1887,6 @@ void *playPlaylist(void *td)
 	maybeplaylist = 0;
 
 	do {
-	    if (result != NULL) {
-		lasterror = result->errorcode;
-		NPN_MemFree(result);
-		result = NULL;
-	    }
 
 	    pthread_testcancel();
 
@@ -1745,6 +1899,8 @@ void *playPlaylist(void *td)
 	    }
 	    argc = base_argc;
 
+	    unEscapeXML(local_url);
+
 	    if (DEBUG)
 		printf("----player thread: playing url: %s\n", local_url);
 	    snprintf(message, 1024, _("Playing %s"), local_list->url);
@@ -1773,7 +1929,7 @@ void *playPlaylist(void *td)
 	    }
 
 	    if (local_td->instance->rtsp_use_tcp == 1) {
-	    	if (strncmp(local_list->url, "rtsp", 4) == 0) {
+		if (strncmp(local_list->url, "rtsp", 4) == 0) {
 		    snprintf(buffer, 1024, "-rtsp-stream-over-tcp");
 		    local_td->argv[argc++] = strdup(buffer);
 		}
@@ -1793,6 +1949,48 @@ void *playPlaylist(void *td)
 //              nomouseinput = 0;
 	    }
 
+	    if (strcmp(local_url, "tv://") == 0) {
+		if (local_td->instance->tv_driver) {
+		    snprintf(buffer, 1024, "-tv:driver");
+		    local_td->argv[argc++] = strdup(buffer);
+		    snprintf(buffer, 1024, "%s",
+			     local_td->instance->tv_driver);
+		    local_td->argv[argc++] = strdup(buffer);
+		}
+
+		if (local_td->instance->tv_input) {
+		    snprintf(buffer, 1024, "-tv:input");
+		    local_td->argv[argc++] = strdup(buffer);
+		    snprintf(buffer, 1024, "%s",
+			     local_td->instance->tv_input);
+		    local_td->argv[argc++] = strdup(buffer);
+		}
+
+		if (local_td->instance->tv_width > 0) {
+		    snprintf(buffer, 1024, "-tv:width");
+		    local_td->argv[argc++] = strdup(buffer);
+		    snprintf(buffer, 1024, "%i",
+			     local_td->instance->tv_width);
+		    local_td->argv[argc++] = strdup(buffer);
+		}
+
+		if (local_td->instance->tv_height > 0) {
+		    snprintf(buffer, 1024, "-tv:height");
+		    local_td->argv[argc++] = strdup(buffer);
+		    snprintf(buffer, 1024, "%i",
+			     local_td->instance->tv_height);
+		    local_td->argv[argc++] = strdup(buffer);
+		}
+
+		if (local_td->instance->tv_outfmt) {
+		    snprintf(buffer, 1024, "-tv:outfmt");
+		    local_td->argv[argc++] = strdup(buffer);
+		    snprintf(buffer, 1024, "%s",
+			     local_td->instance->tv_outfmt);
+		    local_td->argv[argc++] = strdup(buffer);
+		}
+	    }
+
 	    if (local_td->instance->starttime > 0) {
 		local_td->argv[argc++] = strdup("-ss");
 		snprintf(message, 1024, "%li",
@@ -1808,13 +2006,27 @@ void *playPlaylist(void *td)
 #ifdef GTK_ENABLED
 		g_idle_add(gtkgui_updatebuttons, local_td->instance);
 #endif
-		local_td->argv[argc++] = strdup("-cache");
-		snprintf(message, 1024, "%i",
-			 local_td->instance->cachesize);
-		local_td->argv[argc++] = strdup(message);
-
-	    	mmsToHttp(local_url,local_list->url);
+		if (local_td->instance->nomediacache == 0) {
+		    local_td->argv[argc++] = strdup("-cache");
+		    snprintf(message, 1024, "%i",
+			     local_td->instance->cachesize);
+		    local_td->argv[argc++] = strdup(message);
+		}
+		if (DEBUG)
+		    printf("pre local_url is %s\n", local_url);
+		if (result != NULL) {
+		    if (result->tryagain != TRYAGAIN_FALLBACK) {
+			// mmsToHttp(local_url, local_list->url);
+		    }
+		} else {
+		    if (DEBUG)
+			printf("result = null\n");
+		    // mmsToHttp(local_url, local_list->url);
+		}
+		if (DEBUG)
+		    printf("local_url is %s\n", local_url);
 		local_td->argv[argc++] = strdup(local_url);
+		local_list->prefer_url = 1;
 		if (local_td->instance->keep_download) {
 		    snprintf(mmsplaylist, sizeof(mmsplaylist),
 			     "%s/playlist",
@@ -1826,19 +2038,43 @@ void *playPlaylist(void *td)
 		    }
 		}
 	    } else {
-		if (strlen(local_list->fname) == 0) {
-		    if (local_list->playlist == 1 || maybeplaylist != 0)
+		if (result != NULL) {
+		    if (result->errorcode == ERROR_USE_URL) {
+			force_url = 1;
+			local_list->prefer_url = 1;
+		    } else {
+			force_url = 0;
+		    }
+		}
+
+		if (strlen(local_list->fname) == 0 || force_url == 1) {
+		    if (local_td->instance->nomediacache == 0) {
+			local_td->argv[argc++] = strdup("-cache");
+			snprintf(message, 1024, "%i",
+				 local_td->instance->cachesize);
+			local_td->argv[argc++] = strdup(message);
+		    }
+		    if (local_list->playlist == 1 || maybeplaylist != 0) {
+			local_td->argv[argc++] = strdup("-cache");
+			snprintf(message, 1024, "%i",
+				 local_td->instance->cachesize);
+			local_td->argv[argc++] = strdup(message);
 			local_td->argv[argc++] = strdup("-playlist");
+		    }
 		    local_td->argv[argc++] = strdup(local_url);
+		    local_list->prefer_url = 1;
 		} else {
-		    local_td->argv[argc++] = strdup("-nocache");
+//		    local_td->argv[argc++] = strdup("-nocache");
+                    local_td->argv[argc++] = strdup("-cache");
+                    snprintf(message, 1024, "%i",
+                                local_td->instance->cachesize);
+                    local_td->argv[argc++] = strdup(message);
 		    if (local_list->playlist == 1 || maybeplaylist != 0)
 			local_td->argv[argc++] = strdup("-playlist");
 		    local_td->argv[argc++] = strdup(local_list->fname);
 		}
 	    }
 
-
 	    if (DEBUG) {
 		printf("----player thread: URL: %s\n", local_url);
 	    }
@@ -1847,6 +2083,13 @@ void *playPlaylist(void *td)
 	    pthread_cleanup_pop(0);
 
 	    pthread_testcancel();
+#ifdef GTK_ENABLED
+
+		while (gtk_events_pending())
+		    gtk_main_iteration();
+
+#endif
+
 
 	    //start up mplayer
 	    pthread_cleanup_push((void (*)(void *))
@@ -1866,6 +2109,12 @@ void *playPlaylist(void *td)
 	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
 	    pthread_cleanup_pop(0);
 
+	    if (result != NULL) {
+		lasterror = result->errorcode;
+		NPN_MemFree(result);
+		result = NULL;
+	    }
+
 	    local_td->instance->state = STATE_PLAYING;
 	    if (local_td->instance->player != NULL) {
 
@@ -1904,18 +2153,32 @@ void *playPlaylist(void *td)
 		    g_idle_add(gtkgui_draw, local_td->instance);
 #endif
 		}
-		
 		// this cancels the download of this media
 		// if there are more than 1, saves bandwidth
 		// also prevents us from playing it again
 		pthread_mutex_lock(&(local_td->instance->playlist_mutex));
-		if (!(result->retval)) {
-		    local_list->played = 1;
+		if (result->retval == 0) {
+		    if (local_list->mmsstream
+			&& (result->errorcode == ERROR_QUIT)) {
+			local_list->played = 0;
+		    } else {
+			if (result->errorcode == ERROR_FILE_NOT_FOUND) {
+			    // if we got a file not found, don't mark the whole 
+			    // mirror list as not found
+			    local_list->played = 1;
+			} else {
+			    markPlayed(local_list);
+			}
+			//local_list->played = 1;
+		    }
 		}
 		pthread_mutex_unlock(&
 				     (local_td->instance->playlist_mutex));
 
 		assert(local_td->instance != NULL);
+		if (DEBUG)
+		    printf("----player thread: played = %i - %s\n",
+			   local_list->played, local_list->fname);
 
 		//close the pipes to mplayer
 
@@ -1960,9 +2223,20 @@ void *playPlaylist(void *td)
 			   local_td->instance->
 			   mediaCompleteWithErrorCallback);
 	    }
-	
+
 	} while (result->retval);
 
+	// if we are in a list of mmsstreams, then we need to stop 
+
+	if (DEBUG)
+	    printf("mmsstream = %i errorcode = %i \n",
+		   local_list->mmsstream, result->errorcode);
+
+	if (local_list->mmsstream && (result->errorcode == ERROR_QUIT)) {
+	    if (DEBUG)
+		printf("breaking out of next item loop\n");
+	    break;
+	}
 	local_td->instance->currentnode = NULL;
 
 	//if we get here, either the control pipes are closed
