@@ -39,6 +39,7 @@
 #include "nsIMemory.h"
 #include "nsISupportsUtils.h"	// this is where some useful macros defined
 #include <errno.h>
+#include <fcntl.h>
 
 #ifdef GTK2_ENABLED
 #include "../pixmaps/gtk_logo.h"
@@ -125,7 +126,14 @@ nsPluginInstanceBase *NS_NewPluginInstance(nsPluginCreateData *
 	new nsPluginInstance(aCreateDataStruct->instance);
 
     New(plugin, aCreateDataStruct);
-    return plugin;    
+
+    // This line makes the plugin "windowless"
+    // problem is there is no window now to draw too
+    // Also according to 
+    // http://devedge-temp.mozilla.org/library/manuals/2002/plugin/1.0/drawevent.html
+    // X can't do windowless plugins, but that is old
+    // NPN_SetValue(aCreateDataStruct->instance,NPPVpluginWindowBool, FALSE);
+    return plugin;
 }
 
 void NS_DestroyPluginInstance(nsPluginInstanceBase * aPlugin)
@@ -166,8 +174,8 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     window_height = 0;
     embed_width = 0;
     embed_height = 0;
-    movie_width = 0;
-    movie_height = 0;
+    movie_width = 1;
+    movie_height = 1;
     setwindow = 0;
     baseurl = NULL;
     hostname = NULL;
@@ -231,6 +239,9 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     nopauseonhide = 0;
     targetplayer = 0;
     hidestatus = 0;
+    enablecontextmenu = 1;
+    dontparent = 0;
+    volume = -1;
 
 #ifdef GTK_ENABLED
     image = NULL;
@@ -242,6 +253,8 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     stop_event_box = NULL;
     ff_event_box = NULL;
     rew_event_box = NULL;
+    vol_event_box = NULL;
+    vol_tooltip = NULL;
     fs_event_box = NULL;
     src_event_box = NULL;
     gtkwidget = NULL;
@@ -249,6 +262,7 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     drawing_area = NULL;
     popup_menu = NULL;
     conf_window = NULL;
+    file_selector = NULL;
 #ifdef GTK2_ENABLED
     fs_window = NULL;
     paused_wheninvisible = 0;
@@ -285,12 +299,13 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     download_dir = strdup(getenv("HOME"));
     cachesize = 512;
     output_display = NULL;
-    osdlevel = 0;
+    osdlevel = 1;
     qt_speed = SPEED_MED;
     cache_percent = 25;
     framedrop = 0;
     autosync = 0;
     mc = 0;
+    softvol = 1;
     useragent = NULL;
     enable_smil = 1;
     enable_helix = 1;
@@ -303,6 +318,12 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     enable_ogg = 1;
     enable_midi = 0;
     enable_pls = 1;
+    tv_driver = NULL;
+    tv_input = NULL;
+    tv_width = 0;
+    tv_height = 0;
+    tv_outfmt = NULL;
+    profile = NULL;
 
     // JavaScript
     paused = 0;
@@ -384,6 +405,7 @@ void nsPluginInstance::shut()
 void nsPluginInstance::shutdown()
 {
     int i;
+    long flags;
 
     if (DEBUG)
 	printf("shutdown called\n");
@@ -400,6 +422,14 @@ void nsPluginInstance::shutdown()
     if (threadsetup == 1) {
 
 	if (threadlaunched == 1) {
+
+	    // switch to non blocking mode
+	    if (player != NULL) {
+		flags = fcntl(fileno(player), F_GETFL, 0);
+		flags |= O_NONBLOCK;
+		fcntl(fileno(player), F_SETFL, flags);
+	    }
+
 	    pthread_mutex_lock(&control_mutex);
 	    if (paused == 1) {
 		sendCommand(this, "pause\n");
@@ -454,10 +484,12 @@ void nsPluginInstance::shutdown()
 		gtk_widget_destroy(conf_window);
 	}
 	if (targetplayer != 0) {
-	    g_signal_handler_disconnect(GTK_OBJECT(gtkwidget),
-					delete_signal_id);
-	    g_signal_handler_disconnect(GTK_OBJECT(gtkwidget),
-					visible_signal_id);
+	    if (GTK_IS_WIDGET(gtkwidget)) {
+		g_signal_handler_disconnect(GTK_OBJECT(gtkwidget),
+					    delete_signal_id);
+		g_signal_handler_disconnect(GTK_OBJECT(gtkwidget),
+					    visible_signal_id);
+	    }
 	    if (GTK_IS_WIDGET(button_window))
 		gtk_widget_destroy(button_window);
 
@@ -565,11 +597,34 @@ void nsPluginInstance::shutdown()
 	output_display = NULL;
     }
 
+    if (tv_driver != NULL) {
+	free(tv_driver);
+	tv_driver = NULL;
+    }
+
+    if (tv_input != NULL) {
+	free(tv_input);
+	tv_input = NULL;
+
+    }
+
+    if (tv_outfmt != NULL) {
+	free(tv_outfmt);
+	tv_outfmt = NULL;
+    }
+
+    if (profile != NULL) {
+	free(profile);
+	profile = NULL;
+    }
+
+
     nQtNext = 0;
     for (i = 0; i < 256; i++) {
 	if (qtNext[i] != NULL)
 	    free(qtNext[i]);
 	qtNext[i] = NULL;
+
     }
 
     if (download_dir != NULL) {
@@ -733,8 +788,13 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 #ifdef GTK_ENABLED
 	gtkplug = gtk_plug_new(window);
+
 	if (targetplayer == 0) {
-	    gtkwidget = gtk_window_new(GTK_WINDOW_POPUP);
+	    if (dontparent == 0) {
+		gtkwidget = gtk_window_new(GTK_WINDOW_POPUP);
+	    } else {
+		gtkwidget = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	    }
 	} else {
 	    gtkwidget = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	    button_window = gtk_window_new(GTK_WINDOW_POPUP);
@@ -757,11 +817,13 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 	gtk_widget_realize(gtkwidget);
 	gdk_flush();
+
 	gtk_widget_add_events(gtkwidget, GDK_BUTTON_PRESS_MASK);
 	gtk_widget_add_events(gtkwidget, GDK_BUTTON_RELEASE_MASK);
 	gtk_widget_add_events(gtkwidget, GDK_ENTER_NOTIFY_MASK);
 	gtk_widget_add_events(gtkwidget, GDK_LEAVE_NOTIFY_MASK);
 	gtk_widget_add_events(gtkwidget, GDK_KEY_PRESS_MASK);
+	gtk_widget_add_events(gtkwidget, GDK_VISIBILITY_NOTIFY_MASK);
 
 	if (targetplayer == 1) {
 	    window_width = 400;
@@ -780,7 +842,6 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #ifdef GTK1_ENABLED
 	gtk_widget_set_usize(gtkwidget, window_width, window_height);
 #endif
-
 	popup_menu = GTK_MENU(gtk_menu_new());
 	menuitem_play =
 	    GTK_MENU_ITEM(gtk_menu_item_new_with_label(_("Play")));
@@ -844,10 +905,15 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #endif
 
 #ifdef GTK2_ENABLED
-	g_signal_connect_swapped(G_OBJECT(gtkwidget),
-				 "button_press_event",
-				 G_CALLBACK(popup_handler),
-				 GTK_OBJECT(popup_menu));
+	if (enablecontextmenu != 0) {
+	    g_signal_connect_swapped(G_OBJECT(gtkwidget),
+				     "button_press_event",
+				     G_CALLBACK(popup_handler),
+				     GTK_OBJECT(popup_menu));
+	}
+	g_signal_connect(G_OBJECT(gtkwidget),
+			 "key_press_event",
+			 G_CALLBACK(keyboard_callback), this);
 	g_signal_connect(GTK_OBJECT(menuitem_play), "activate",
 			 G_CALLBACK(menuitem_play_callback), this);
 	g_signal_connect(GTK_OBJECT(menuitem_pause), "activate",
@@ -864,8 +930,6 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 			 G_CALLBACK(menuitem_copy_callback), this);
 	g_signal_connect(GTK_OBJECT(menuitem_config), "activate",
 			 G_CALLBACK(menuitem_config_callback), this);
-	// code to try and autopause on window covered up, does not appear GTK sends enough events
-	gtk_widget_add_events(gtkwidget, GDK_VISIBILITY_NOTIFY_MASK);
 	g_signal_connect(G_OBJECT(gtkwidget), "button_press_event",
 			 G_CALLBACK(mouse_callback), this);
 	g_signal_connect(G_OBJECT(gtkwidget), "button_release_event",
@@ -882,6 +946,9 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 				  "button_press_event",
 				  GTK_SIGNAL_FUNC(popup_handler),
 				  GTK_OBJECT(popup_menu));
+	gtk_signal_connect(GTK_OBJECT(gtkwidget),
+			   "key_press_event",
+			   GTK_SIGNAL_FUNC(keyboard_callback), this);
 	gtk_signal_connect(GTK_OBJECT(menuitem_play), "activate",
 			   GTK_SIGNAL_FUNC(menuitem_play_callback), this);
 	gtk_signal_connect(GTK_OBJECT(menuitem_pause), "activate",
@@ -907,13 +974,10 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 	fixed_event_box = gtk_event_box_new();
 	fixed_container = gtk_fixed_new();
-	gtk_widget_set_events(GTK_WIDGET(fixed_event_box),
-			      GDK_BUTTON_PRESS_MASK);
+
+
 #ifdef GTK2_ENABLED
-	g_signal_connect_swapped(G_OBJECT(fixed_event_box),
-				 "button_press_event",
-				 G_CALLBACK(popup_handler),
-				 GTK_OBJECT(popup_menu));
+
 	gtk_widget_set_size_request(GTK_WIDGET(fixed_container),
 				    window_width, window_height);
 	gtk_widget_set_size_request(GTK_WIDGET(fixed_event_box),
@@ -921,10 +985,6 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 #endif
 #ifdef GTK1_ENABLED
-	gtk_signal_connect_object(GTK_OBJECT(fixed_event_box),
-				  "button_press_event",
-				  GTK_SIGNAL_FUNC(popup_handler),
-				  GTK_OBJECT(popup_menu));
 	gtk_widget_set_usize(GTK_WIDGET(fixed_container), window_width,
 			     window_height);
 	gtk_widget_set_usize(GTK_WIDGET(fixed_event_box), window_width,
@@ -934,6 +994,8 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	gtk_container_add(GTK_CONTAINER(fixed_event_box), fixed_container);
 
 	status = GTK_LABEL(gtk_label_new(_("Initializing")));
+	// I don't really like this as it messes up the right click menu
+	// gtk_label_set_selectable(status,TRUE);
 	progress_bar = GTK_PROGRESS_BAR(gtk_progress_bar_new());
 
 	if (window_height > 125) {
@@ -971,51 +1033,41 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	    gtk_fixed_put(GTK_FIXED(fixed_container), GTK_WIDGET(status),
 			  10, 60);
 	} else {
-	    if (window_height < 20) {
-		if (window_width < 126) {
-		    gtk_fixed_put(GTK_FIXED(fixed_container),
-				  GTK_WIDGET(status), 65, 0);
-		} else {
-		    gtk_fixed_put(GTK_FIXED(fixed_container),
-				  GTK_WIDGET(status), 105, 0);
-		}
-	    } else {
-		gtk_fixed_put(GTK_FIXED(fixed_container),
-			      GTK_WIDGET(status), 10, 10);
-	    }
+	    // small window layout
+	    gtk_fixed_put(GTK_FIXED(fixed_container),
+			  GTK_WIDGET(status), 0, 0);
 	    gtk_fixed_put(GTK_FIXED(fixed_container),
 			  GTK_WIDGET(progress_bar), 10,
-			  window_height - 20);
+			  window_height - 31);
 	    if (!hidestatus)
 		gtk_widget_show(GTK_WIDGET(progress_bar));
 #ifdef GTK2_ENABLED
 	    gtk_widget_set_size_request(GTK_WIDGET(progress_bar),
 					window_width - 20, 15);
 	    gtk_widget_set_size_request(GTK_WIDGET(status),
-					window_width - 20,
-					window_height - 25);
+					window_width,
+					window_height - 31);
 #endif
 #ifdef GTK1_ENABLED
-	    if (DEBUG > 1)
-		printf("< 125 setup\n");
 	    gtk_widget_set_usize(GTK_WIDGET(progress_bar),
 				 aWindow->width - 20, 15);
 	    gtk_widget_set_usize(GTK_WIDGET(status),
-				 aWindow->width - 20,
-				 aWindow->height - 25);
+				 aWindow->width,
+				 aWindow->height - 31);
 #endif
 	}
 
 	gtk_misc_set_alignment(GTK_MISC(status), 0, 0);
 	gtk_label_set_line_wrap(status, TRUE);
 	drawing_area = gtk_socket_new();
-	gtk_widget_set_events(drawing_area, GDK_BUTTON_PRESS_MASK);
+
 	if (mode == NP_EMBED) {
 	    gtk_fixed_put(GTK_FIXED(fixed_container), drawing_area, 0, 0);
 	} else {
 	    gtk_fixed_put(GTK_FIXED(fixed_container),
 			  drawing_area, 10, 100);
 	}
+	gtk_widget_set_usize(drawing_area, 1, 1);
 	gtk_widget_show(drawing_area);
 
 
@@ -1037,8 +1089,7 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #endif
 
 #ifdef GTK2_ENABLED
-	    gtk_widget_modify_bg(gtkwidget, GTK_STATE_NORMAL,
-				 &black);
+	    gtk_widget_modify_bg(gtkwidget, GTK_STATE_NORMAL, &black);
 	    gtk_widget_modify_bg(fixed_event_box, GTK_STATE_NORMAL,
 				 &black);
 	    gtk_widget_modify_fg(GTK_WIDGET(status), GTK_STATE_NORMAL,
@@ -1055,10 +1106,12 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #ifdef GTK_ENABLED
 
 	if (targetplayer == 0) {
-	    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-			    GDK_WINDOW_XWINDOW(gtkwidget->window), window,
-			    0, 0);
-	    gtk_widget_map(gtkwidget);
+	    if (dontparent == 0) {
+		XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
+				GDK_WINDOW_XWINDOW(gtkwidget->window),
+				window, 0, 0);
+		gtk_widget_map(gtkwidget);
+	    }
 	} else {
 	    gtk_widget_show(button_window);
 	    XReparentWindow(GDK_WINDOW_XDISPLAY(button_window->window),
@@ -1208,9 +1261,13 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 		gtk_widget_set_usize(GTK_WIDGET(mediaprogress_bar),
 				     window_width -
-				     (width * (multiplier + 1) + 20),
+				     (width * (multiplier + 2) + 20),
 				     height - 4);
 	    }
+	    gtk_fixed_move(GTK_FIXED(fixed_container),
+			   GTK_WIDGET(vol_event_box),
+			   (window_width - (width * 2)),
+			   window_height - height);
 	    gtk_fixed_move(GTK_FIXED(fixed_container),
 			   GTK_WIDGET(fs_event_box),
 			   (window_width - width), window_height - height);
@@ -1219,12 +1276,23 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	if (progress_bar != NULL) {
 	    gtk_fixed_move(GTK_FIXED(fixed_container),
 			   GTK_WIDGET(progress_bar),
-			   10, window_height - 50);
+			   10, window_height - 31);
 
 	    gtk_widget_set_usize(GTK_WIDGET(progress_bar),
 				 window_width - 20, 15);
 	}
 
+	if (GTK_IS_WIDGET(drawing_area) && (movie_width > 0)
+	    && (movie_height > 0) && mode == NP_EMBED) {
+	    if (showcontrols) {
+		gtk_widget_set_usize(GTK_WIDGET(drawing_area),
+				     window_width, window_height - 16);
+	    } else {
+		gtk_widget_set_usize(GTK_WIDGET(drawing_area),
+				     window_width, window_height);
+	    }
+	}
+
     }
     if (gtkwidget == NULL)
 	return NPERR_NO_ERROR;
@@ -1317,7 +1385,9 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 
 	if (n != NULL) {
 	    n->retrieved = 1;
-
+#ifdef GTK_ENABLED
+	    g_idle_add(gtkgui_save_enable, this);
+#endif
 	    if (n->localcache != NULL) {
 		if (fclose(n->localcache) != 0) {
 		    if (DEBUG)
@@ -1351,7 +1421,7 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 			NPN_GetURL(mInstance, href, NULL);
 			return NPERR_NO_ERROR;
 		    }
-		} else if ( strstr(n->url, url) && href != NULL) {
+		} else if (strstr(n->url, url) && href != NULL) {
 		    // we are in this block because the url is a relative url and so
 		    // it is a subset of the fully qualified n->url
 		    if (srcToButton(n->fname, this)) {
@@ -1611,6 +1681,7 @@ void nsPluginInstance::URLNotify(const char *url, NPReason reason,
 int32 nsPluginInstance::WriteReady(NPStream * stream)
 {
     Node *n;
+    char *filename;
 
     if (state == STATE_PLAY_CANCELLED || cancelled == 1)
 	return -1;
@@ -1647,6 +1718,28 @@ int32 nsPluginInstance::WriteReady(NPStream * stream)
 	n = n->next;
     }
 
+    // redirect case on initial node
+    if (n == NULL) {
+	if (DEBUG)
+	    printf("n == NULL\n");
+	if (td->list != NULL) {
+	    if (DEBUG)
+		printf("td->list != NULL\n");
+	    if (((strlen(td->list->fname) == 0)
+		 ||
+		 (strncmp
+		  (getURLFilename(td->list->url),
+		   getURLFilename(stream->url), 1024) == 0))
+		&& (mmsstream == 0)) {
+		if (DEBUG)
+		    printf("Redirected initial URL\n");
+		n = list;
+		snprintf(n->url, 1024, "%s", stream->url);
+	    }
+	}
+    }
+
+
     if (n != NULL) {
 	// if we have determined that we are not going to play the file
 	// stop downloading it, and clean it up during cleanup
@@ -1685,17 +1778,34 @@ int32 nsPluginInstance::WriteReady(NPStream * stream)
 	    return -1;
 	} else {
 	    if (strlen(n->fname) == 0) {
-		snprintf(n->fname, 1024, "%s",
-			 tempnam("/tmp", "mplayerplug-inXXXXXX"));
-		if (strstr(mimetype, "midi") != NULL) {
-		    strlcat(n->fname, ".mid", 1024);
-		}
-		if (strstr(mimetype, "mp3") != NULL) {
-		    strlcat(n->fname, ".mp3", 1024);
-		}
-		if (strstr(mimetype, "audio/mpeg") != NULL) {
-		    strlcat(n->fname, ".mp3", 1024);
+		if (keep_download == 1) {
+		    n->remove = 0;
+		    filename = getURLFilename(n->url);
+		    snprintf(n->fname, 1024, "%s/%s",
+			     download_dir, filename);
+		    if (filename)
+			NPN_MemFree(filename);
+		} else {
+		    snprintf(n->fname, 1024, "%s",
+			     tempnam("/tmp", "mplayerplug-inXXXXXX"));
+		    if (strstr(mimetype, "midi") != NULL) {
+			strlcat(n->fname, ".mid", 1024);
+		    }
+		    if (strstr(mimetype, "mp3") != NULL) {
+			strlcat(n->fname, ".mp3", 1024);
+		    }
+		    if (strstr(mimetype, "audio/mpeg") != NULL) {
+			strlcat(n->fname, ".mp3", 1024);
+		    }
+		    if (strstr(mimetype, "audio/x-mod") != NULL) {
+			strlcat(n->fname, ".mod", 1024);
+		    }
+		    if (strstr(mimetype, "flac") != NULL) {
+			strlcat(n->fname, ".flac", 1024);
+		    }
+
 		}
+
 		if (DEBUG)
 		    printf("WR tempname: %s\n", n->fname);
 
@@ -1736,8 +1846,16 @@ int32 nsPluginInstance::WriteReady(NPStream * stream)
 	    }
 	    return -1;
 	} else {
-	    snprintf(n->fname, 1024, "%s",
-		     tempnam("/tmp", "mplayerplug-inXXXXXX"));
+	    if (keep_download == 1) {
+		n->remove = 0;
+		filename = getURLFilename(n->url);
+		snprintf(n->fname, 1024, "%s/%s", download_dir, filename);
+		if (filename)
+		    NPN_MemFree(filename);
+	    } else {
+		snprintf(n->fname, 1024, "%s",
+			 tempnam("/tmp", "mplayerplug-inXXXXXX"));
+	    }
 	    addToEnd(td->list, n);
 	    if (n->totalbytes != (int) stream->end)
 		n->totalbytes = stream->end;
@@ -1839,7 +1957,7 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 		// normal media so keep going.
 
 		if (n->localcache == NULL) {
-		    fp = fopen(n->fname, "a+");
+		    fp = fopen(n->fname, "w+");
 		    n->localcache = fp;
 		} else {
 		    fp = n->localcache;
@@ -1863,10 +1981,15 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 					((double) currdownload /
 					 (double) maxdownload)),
 				 (n->bytes / 1024));
+			percent = ((double) currdownload /
+				   (double) maxdownload);
+
 		    } else {
 			snprintf(message, 1024,
 				 _("Buffering Complete - %li KB"),
 				 (n->bytes / 1024));
+			percent = 1.0;
+			n->retrieved = 1;
 		    }
 		}
 		if (mode == NP_EMBED && noembed == 0 && fullscreen == 0) {
@@ -1960,6 +2083,7 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 	    if (n->play == 1) {
 		if (n->localcache != NULL && nomediacache == 1) {
 		    fclose(n->localcache);
+		    n->localcache = NULL;
 		}
 		if (n->mmsstream == 1) {
 		    if (threadsignaled == 0) {
@@ -2019,6 +2143,7 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 void nsPluginInstance::Play()
 {
     Node *n;
+    int clearlist = 1;
 
     if (DEBUG > 1) {
 	printf("*****Play Called\n");
@@ -2033,12 +2158,22 @@ void nsPluginInstance::Play()
 	    printf("Play: resetting playlist\n");
 
 	pthread_mutex_lock(&playlist_mutex);	// manipulating the playlist, so lock it
+
 	n = list;
 	while (n != NULL) {
-	    if (n->played == 1)
-		n->played = 0;	// reset played flag
+	    if (n->played == 0 && n->play == 1) {
+		clearlist = 0;
+	    }
 	    n = n->next;
 	}
+	if (clearlist) {
+	    n = list;
+	    while (n != NULL) {
+		if (n->played == 1)
+		    n->played = 0;	// reset played flag
+		n = n->next;
+	    }
+	}
 	pthread_mutex_unlock(&playlist_mutex);	// unlock the playlist
     }
     // the thread is not inited
@@ -2068,7 +2203,8 @@ void nsPluginInstance::Play()
 	if (DEBUG > 1)
 	    printf("Play: ready to signal\n");
 	while (state < STATE_WAITING_FOR_SIGNAL) {
-	    printf("sleeping state = %i\n", state);
+	    if (DEBUG > 1)
+		printf("sleeping state = %i\n", state);
 	    usleep(100);
 	}
 	if (DEBUG > 1)
@@ -2240,6 +2376,53 @@ void nsPluginInstance::Seek(double counter)
 
 }
 
+void nsPluginInstance::VolumeUp()
+{
+    char command[32];
+
+    if (threadlaunched == 0)
+	return;
+    pthread_mutex_lock(&control_mutex);
+    snprintf(command, 32, "volume +1\n");
+    sendCommand(this, command);
+    sendCommand(this, "get_property volume\n");
+    pthread_mutex_unlock(&control_mutex);
+
+}
+
+void nsPluginInstance::VolumeDown()
+{
+    char command[32];
+
+    if (threadlaunched == 0)
+	return;
+    pthread_mutex_lock(&control_mutex);
+    snprintf(command, 32, "volume -1\n");
+    sendCommand(this, command);
+    sendCommand(this, "get_property volume\n");
+    pthread_mutex_unlock(&control_mutex);
+
+}
+
+void nsPluginInstance::GetVolume(double *_retval)
+{
+    *_retval = (double) volume;
+
+}
+
+void nsPluginInstance::SetVolume(double value)
+{
+    volume = (int) value;
+    char command[32];
+
+    if (threadlaunched == 0)
+	return;
+    pthread_mutex_lock(&control_mutex);
+    snprintf(command, 32, "volume %i 1\n", volume);
+    sendCommand(this, command);
+    pthread_mutex_unlock(&control_mutex);
+}
+
 void nsPluginInstance::GetPlayState(PRInt32 * playstate)
 {
     pthread_mutex_lock(&control_mutex);
@@ -2391,6 +2574,8 @@ void nsPluginInstance::SetShowControls(PRBool value)
 		gtk_widget_show(rew_event_box);
 	    if (mediaprogress_bar != NULL && mediaPercent > 0)
 		gtk_widget_show(GTK_WIDGET(mediaprogress_bar));
+	    if (vol_event_box != NULL)
+		gtk_widget_show(GTK_WIDGET(vol_event_box));
 	    if (fs_event_box != NULL)
 		gtk_widget_show(GTK_WIDGET(fs_event_box));
 	}
@@ -2413,6 +2598,8 @@ void nsPluginInstance::SetShowControls(PRBool value)
 		gtk_widget_hide(rew_event_box);
 	    if (mediaprogress_bar != NULL)
 		gtk_widget_hide(GTK_WIDGET(mediaprogress_bar));
+	    if (vol_event_box != NULL)
+		gtk_widget_hide(GTK_WIDGET(vol_event_box));
 	    if (fs_event_box != NULL)
 		gtk_widget_hide(GTK_WIDGET(fs_event_box));
 	}
@@ -2501,18 +2688,22 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 	    gtk_window_get_size(GTK_WINDOW(gtkwidget), &wwidth, &wheight);
 	    if ((wwidth != rect.width) && (wheight != rect.height)) {
 		if (targetplayer == 0) {
-		    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-				    GDK_WINDOW_XWINDOW(gtkwidget->window),
-				    GDK_WINDOW_XWINDOW(fs_window->window),
-				    0, 0);
+		    if (dontparent == 0) {
+			XReparentWindow(GDK_WINDOW_XDISPLAY
+					(gtkwidget->window),
+					GDK_WINDOW_XWINDOW(gtkwidget->
+							   window),
+					GDK_WINDOW_XWINDOW(fs_window->
+							   window), 0, 0);
 
-		    gtk_widget_map(gtkwidget);
+			gtk_widget_map(gtkwidget);
 
-		    gtk_window_resize(GTK_WINDOW(gtkwidget), rect.width,
-				      rect.height);
-		    if (DEBUG)
-			printf("gtkwidget resized to %i x %i\n",
-			       rect.width, rect.height);
+			gtk_window_resize(GTK_WINDOW(gtkwidget),
+					  rect.width, rect.height);
+			if (DEBUG)
+			    printf("gtkwidget resized to %i x %i\n",
+				   rect.width, rect.height);
+		    }
 		}
 	    } else {
 		if (DEBUG)
@@ -2558,10 +2749,14 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 	    if (fs_window != NULL) {
 		gtk_window_unfullscreen(GTK_WINDOW(fs_window));
 		if (targetplayer == 0) {
-		    XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-				    GDK_WINDOW_XWINDOW(gtkwidget->window),
-				    window, 0, 0);
-		    gtk_widget_map(gtkwidget);
+		    if (dontparent == 0) {
+			XReparentWindow(GDK_WINDOW_XDISPLAY
+					(gtkwidget->window),
+					GDK_WINDOW_XWINDOW(gtkwidget->
+							   window), window,
+					0, 0);
+			gtk_widget_map(gtkwidget);
+		    }
 		}
 		gtk_widget_destroy(fs_window);
 		fs_window = NULL;
@@ -2629,14 +2824,17 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 				   button_top + 2);
 		    gtk_widget_set_usize(GTK_WIDGET(mediaprogress_bar),
 					 win_width -
-					 (width * (multiplier + 1) + 20),
+					 (width * (multiplier + 2) + 20),
 					 height - 4);
 		}
-		if (win_width > 126)
+		if (win_width > 126) {
+		    gtk_fixed_move(GTK_FIXED(fixed_container),
+				   GTK_WIDGET(vol_event_box),
+				   (win_width - (width * 2)), button_top);
 		    gtk_fixed_move(GTK_FIXED(fixed_container),
 				   GTK_WIDGET(fs_event_box),
 				   (win_width - width), button_top);
-
+		}
 
 	    }
 
@@ -2740,7 +2938,7 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 
 		gtk_widget_show(fs_window);
 		gtk_window_fullscreen(GTK_WINDOW(fs_window));
-		gtk_widget_reparent(gtkwidget, fs_window);
+		//gtk_widget_reparent(gtkwidget, fs_window);
 		XReparentWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
 				GDK_WINDOW_XWINDOW(gtkwidget->window),
 				GDK_WINDOW_XWINDOW(fs_window->window), 0,
@@ -2832,9 +3030,12 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 				   y - height + 2);
 
 		    gtk_widget_set_usize(GTK_WIDGET(mediaprogress_bar),
-					 x - (width * (multiplier + 1) +
+					 x - (width * (multiplier + 2) +
 					      20), height - 4);
 		}
+		gtk_fixed_move(GTK_FIXED(fixed_container),
+			       GTK_WIDGET(vol_event_box),
+			       (x - (width * 2)), y - height);
 		gtk_fixed_move(GTK_FIXED(fixed_container),
 			       GTK_WIDGET(fs_event_box),
 			       (x - width), y - height);
@@ -2986,7 +3187,7 @@ void nsPluginInstance::PlaylistAppend(const char *item)
 	    tail = tail->next;
     }
     fullyQualifyURL(this, (char *) item, localurl);
-    addToList(this, localurl, tail, -1);
+    addToList(this, localurl, tail, -1, 0, -1,0);
     pthread_mutex_unlock(&playlist_mutex);
 }
 
