@@ -90,6 +90,10 @@ class nsPluginInstance:public nsPluginInstanceBase {
     void FastForward();
     void FastReverse();
     void Seek(double counter);
+	void VolumeUp();
+	void VolumeDown();
+    void SetVolume(double value);
+    void GetVolume(double * _retval);
     void GetPlayState(PRInt32 * playstate);
     void GetTime(double *_retval);
     void GetDuration(double *_retval);
@@ -202,6 +206,8 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int targetplayer;		// if we have this create a new window and play there
     int hidestatus;
     int enablecontextmenu;
+    int dontparent;
+	int volume;
     
 #ifdef GTK_ENABLED
     gulong delete_signal_id;
@@ -223,14 +229,18 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GtkWidget *stop_event_box;
     GtkWidget *ff_event_box;
     GtkWidget *rew_event_box;
+    GtkWidget *vol_event_box;
     GtkWidget *fs_event_box;
     GtkWidget *src_event_box;
 
+    GtkTooltips *vol_tooltip;
+
     GtkWidget *image_play;
     GtkWidget *image_pause;
     GtkWidget *image_stop;
     GtkWidget *image_ff;
     GtkWidget *image_rew;
+    GtkWidget *image_vol;
     GtkWidget *image_fs;
     GtkWidget *image_src;
 
@@ -295,6 +305,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GdkPixbuf *pb_sm_ff_down;
     GdkPixbuf *pb_sm_rew_up;
     GdkPixbuf *pb_sm_rew_down;
+    GdkPixbuf *pb_sm_vol;
     GdkPixbuf *pb_sm_fs_up;
     GdkPixbuf *pb_sm_fs_down;
     GdkPixbuf *pb_src;
@@ -315,6 +326,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GdkPixmap *pb_sm_ff_down;
     GdkPixmap *pb_sm_rew_up;
     GdkPixmap *pb_sm_rew_down;
+    GdkPixmap *pb_sm_vol;
     GdkPixmap *pb_sm_fs_up;
     GdkPixmap *pb_sm_fs_down;
 
@@ -365,6 +377,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int framedrop;
     int autosync;
     int mc;
+	int softvol;
     char *useragent;
     int enable_smil;
     int enable_helix;
@@ -379,6 +392,13 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int enable_midi;
     int enable_pls;
 
+    char *tv_driver;
+    char *tv_input;
+    int tv_width;
+    int tv_height;
+    char *tv_outfmt;
+	char *profile;
+
     // JavaScript State
     int paused;
     int js_state;
