@@ -9,7 +9,7 @@ libdir=${exec_prefix}/lib
 datadir=${prefix}/share
 LOCALEDIR=${datadir}/locale
 DESTDIR=$1
-MOZHOME=/plugins
+MOZHOME=/usr/lib/firefox-2.0.0.2/plugins
 
 if [ "$DESTDIR" ]; then 
 	echo "Installing to $DESTDIR"
