@@ -1,5 +1,5 @@
 #!/bin/bash
-MOZHOME=
+MOZHOME=/usr/lib/firefox-2.0.0.2
 
 echo "Cleaning up user plugins"
 if [ -f ${HOME}/.mozilla/plugins/mplayerplug-in.so ]; then rm ${HOME}/.mozilla/plugins/mplayerplug-in*; fi
@@ -12,7 +12,6 @@ if [ `whoami` == 'root' ]; then
 	fi
 	if [ -z ${MOZHOME} ]; then echo "Can't find MOZILLA HOME. Please uninstall manually"; exit 1; fi
 	if [ -f ${MOZHOME}/plugins/mplayerplug-in.so ]; then rm ${MOZHOME}/plugins/mplayerplug-in*; fi
-	if [ -f ${MOZHOME}/plugins/mplayerplug-in.xpt ]; then rm ${MOZHOME}/components/mplayerplug-in*; fi
 	if [ -f ${MOZHOME}/components/mplayerplug-in.xpt ]; then rm ${MOZHOME}/components/mplayerplug-in*; fi
 	if [ "x${MOZ}" != "x" ]; then
 	    MOZPLUGINS=`grep MOZ_PLUGIN_PATH ${MOZ} | awk -F = '{ print $2 }' | awk -F : '{ if ( $1 != "") if ( index($1,"_PLUGIN_") == 0) print $1 }'`
